"""Define endpoints for interacting with bridges."""
from typing import Callable


class Bridge:  # pylint: disable=too-few-public-methods
    """Define an object to interact with all endpoints."""

    def __init__(self, request: Callable) -> None:
        """Initialize."""
        self._request: Callable = request

    async def async_all(self) -> list:
        """Get all bridges."""
        resp: dict = await self._request("get", "base_stations")
        return resp["base_stations"]

    async def async_create(self, attributes: dict) -> dict:
        """Create a bridge with a specific attribute payload."""
        resp: dict = await self._request(
            "post", "base_stations", json={"base_stations": attributes}
        )
        return resp["base_stations"]

    async def async_delete(self, bridge_id: int) -> None:
        """Delete a bridge by ID."""
        await self._request("delete", f"base_stations/{bridge_id}")

    async def async_get(self, bridge_id: int) -> dict:
        """Get a bridge by ID."""
        resp: dict = await self._request("get", f"base_stations/{bridge_id}")
        return resp["base_stations"]

    async def async_reset(self, bridge_id: int) -> dict:
        """Reset a bridge (clear its wifi credentials) by ID."""
        resp: dict = await self._request("put", f"base_stations/{bridge_id}/reset")
        return resp["base_stations"]

    async def async_update(self, bridge_id: int, new_attributes: dict) -> dict:
        """Update a bridge with a specific attribute payload."""
        resp: dict = await self._request(
            "put", f"base_stations/{bridge_id}", json={"base_stations": new_attributes}
        )
        return resp["base_stations"]



"""Define fixtures for systems."""
import pytest


@pytest.fixture()
def system_all_json():
    """Define a response to GET /systems."""
    return {
        "systems": [
            {
                "id": 12345,
                "uuid": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
                "name": "Home",
                "mode": "home",
                "latitude": 40.6892494,
                "longitude": -74.0445004,
                "timezone_id": "America/New_York",
                "locality": "New York",
                "postal_code": "10004",
                "administrative_area": "New York",
                "fire_number": "(212) 363-3200",
                "police_number": "(212) 363-3200",
                "emergency_number": "(212) 363-3200",
                "night_time_start": "2019-05-01T04:00:00.000Z",
                "night_time_end": "2019-05-01T13:00:00.000Z",
                "created_at": "2019-04-30T01:35:21.870Z",
                "updated_at": "2019-04-30T01:35:23.723Z",
            }
        ]
    }


@pytest.fixture()
def system_create_json():
    """Define a response to POST /systems."""
    return {
        "systems": {
            "id": 12345,
            "uuid": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            "name": "New System",
            "mode": "home",
            "latitude": 40.6892494,
            "longitude": -74.0445004,
            "timezone_id": "America/New_York",
            "locality": "New York",
            "postal_code": "10004",
            "administrative_area": "New York",
            "fire_number": "(212) 363-3200",
            "police_number": "(212) 363-3200",
            "emergency_number": "(212) 363-3200",
            "night_time_start": "2019-05-01T04:00:00.000Z",
            "night_time_end": "2019-05-01T13:00:00.000Z",
            "created_at": "2019-04-30T01:35:21.870Z",
            "updated_at": "2019-04-30T01:35:23.723Z",
        }
    }


@pytest.fixture()
def system_get_json():
    """Define a response to GET /systems/:id."""
    return {
        "systems": {
            "id": 12345,
            "uuid": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            "name": "Home",
            "mode": "home",
            "latitude": 40.6892494,
            "longitude": -74.0445004,
            "timezone_id": "America/New_York",
            "locality": "New York",
            "postal_code": "10004",
            "administrative_area": "New York",
            "fire_number": "(212) 363-3200",
            "police_number": "(212) 363-3200",
            "emergency_number": "(212) 363-3200",
            "night_time_start": "2019-05-01T04:00:00.000Z",
            "night_time_end": "2019-05-01T13:00:00.000Z",
            "created_at": "2019-04-30T01:35:21.870Z",
            "updated_at": "2019-04-30T01:35:23.723Z",
        }
    }


@pytest.fixture()
def system_update_json():
    """Define a response to PUT /systems/:id."""
    return {
        "systems": {
            "id": 12345,
            "uuid": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            "name": "Updated System Name",
            "mode": "home",
            "latitude": 40.6892494,
            "longitude": -74.0445004,
            "timezone_id": "America/New_York",
            "locality": "New York",
            "postal_code": "10004",
            "administrative_area": "New York",
            "fire_number": "(212) 363-3200",
            "police_number": "(212) 363-3200",
            "emergency_number": "(212) 363-3200",
            "night_time_start": "2019-05-01T04:00:00.000Z",
            "night_time_end": "2019-05-01T13:00:00.000Z",
            "created_at": "2019-04-30T01:35:21.870Z",
            "updated_at": "2019-04-30T01:35:23.723Z",
        }
    }



"""Define fixtures for devices."""
import pytest


@pytest.fixture()
def device_all_json():
    """Define a response to GET /devices."""
    return {
        "devices": [
            {
                "id": 12345,
                "token": "123456abcde",
                "platform": "ios",
                "endpoint": "arn:aws:sns:us-west-2:307936840629:...",
                "created_at": "2019-06-17T00:57:09.937Z",
                "updated_at": "2019-06-17T00:57:09.937Z",
            }
        ]
    }


@pytest.fixture()
def device_create_json():
    """Define a response to POST /devices."""
    return {
        "devices": {
            "id": 12345,
            "token": "123456abcde",
            "platform": "ios",
            "endpoint": "arn:aws:sns:us-west-2:307936840629:...",
            "created_at": "2019-06-17T00:57:09.937Z",
            "updated_at": "2019-06-17T00:57:09.937Z",
        }
    }


@pytest.fixture()
def device_get_json():
    """Define a response to GET /devices/:id."""
    return {
        "devices": {
            "id": 12345,
            "token": "123456abcde",
            "platform": "ios",
            "endpoint": "arn:aws:sns:us-west-2:307936840629:...",
            "created_at": "2019-06-17T00:57:09.937Z",
            "updated_at": "2019-06-17T00:57:09.937Z",
        }
    }



"""Define endpoints for interacting with tasks (monitored conditions)."""
from datetime import datetime
from typing import Callable, List


class Task:
    """Define an object to interact with all endpoints."""

    def __init__(self, request: Callable) -> None:
        """Initialize."""
        self._request: Callable = request

    async def async_all(self) -> list:
        """Get all tasks."""
        resp: dict = await self._request("get", "tasks")
        return resp["tasks"]

    async def async_create(self, sensor_id: int, tasks: List[dict]) -> list:
        """Create new tasks based upon a list of attribute dicts."""
        resp: dict = await self._request(
            "post",
            f"sensors/{sensor_id}/tasks",
            json={"sensor_id": sensor_id, "tasks": tasks},
        )
        return resp["tasks"]

    async def async_delete(self, sensor_id: int, task_id: str) -> None:
        """Delete a task by ID."""
        await self._request("delete", f"sensors/{sensor_id}/tasks/{task_id}")

    async def async_get(self, task_id: str) -> dict:
        """Get a task by ID."""
        resp: dict = await self._request("get", f"tasks/{task_id}")
        return resp["tasks"]

    async def async_history(
        self, task_id: str, data_before: datetime, data_after: datetime
    ) -> dict:
        """Get the history of a task's values between two datetimes."""
        resp: dict = await self._request(
            "get",
            f"tasks/{task_id}/data",
            params={
                "data_before": data_before.isoformat(),
                "data_after": data_after.isoformat(),
            },
        )
        return resp["task"]["data"]



"""Define fixtures for testing the Notion API."""
import pytest

from ..const import TEST_EMAIL, TEST_TOKEN


@pytest.fixture()
def auth_failure_json():
    """Define a failed response to GET /users/sign_in."""
    return {"errors": [{"title": "Invalid email or password."}]}


@pytest.fixture()
def auth_success_json():
    """Define a successful response to GET /users/sign_in."""
    return {
        "users": {
            "id": 12345,
            "uuid": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            "first_name": "John",
            "last_name": "Doe",
            "email": TEST_EMAIL,
            "phone_number": None,
            "role": "user",
            "organization": "Notion User",
            "authentication_token": TEST_TOKEN,
            "created_at": "2019-04-30T01:35:03.781Z",
            "updated_at": "2019-04-30T17:40:21.030Z",
        },
        "session": {
            "user_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            "authentication_token": TEST_TOKEN,
        },
    }


@pytest.fixture()
def bad_api_json():
    """Define a response to GET /bad_endpoint."""
    return {"errors": [{"title": "No records found"}]}



"""Define fixtures for bridges."""
import pytest


@pytest.fixture()
def bridge_all_json():
    """Define a response to GET /base_stations."""
    return {
        "base_stations": [
            {
                "id": 12345,
                "name": None,
                "mode": "home",
                "hardware_id": "0x1234567890abcdef",
                "hardware_revision": 4,
                "firmware_version": {
                    "wifi": "0.121.0",
                    "wifi_app": "3.3.0",
                    "silabs": "1.0.1",
                },
                "missing_at": None,
                "created_at": "2019-04-30T01:43:50.497Z",
                "updated_at": "2019-04-30T01:44:43.749Z",
                "system_id": 12345,
                "firmware": {"wifi": "0.121.0", "wifi_app": "3.3.0", "silabs": "1.0.1"},
                "links": {"system": 12345},
            }
        ]
    }


@pytest.fixture()
def bridge_create_json():
    """Define a response to POST /base_stations."""
    return {
        "base_stations": {
            "id": 98765,
            "name": "New Bridge",
            "mode": "home",
            "hardware_id": "0x1234567890abcdef",
            "hardware_revision": 4,
            "firmware_version": {
                "wifi": "0.121.0",
                "wifi_app": "3.3.0",
                "silabs": "1.0.1",
            },
            "missing_at": None,
            "created_at": "2019-04-30T01:43:50.497Z",
            "updated_at": "2019-04-30T01:44:43.749Z",
            "system_id": 12345,
            "firmware": {"wifi": "0.121.0", "wifi_app": "3.3.0", "silabs": "1.0.1"},
            "links": {"system": 12345},
        }
    }


@pytest.fixture()
def bridge_get_json():
    """Define a response to GET /base_stations/:id."""
    return {
        "base_stations": {
            "id": 12345,
            "name": "My Bridge",
            "mode": "home",
            "hardware_id": "0x1234567890abcdef",
            "hardware_revision": 4,
            "firmware_version": {
                "wifi": "0.121.0",
                "wifi_app": "3.3.0",
                "silabs": "1.0.1",
            },
            "missing_at": None,
            "created_at": "2019-04-30T01:43:50.497Z",
            "updated_at": "2019-04-30T01:44:43.749Z",
            "system_id": 12345,
            "firmware": {"wifi": "0.121.0", "wifi_app": "3.3.0", "silabs": "1.0.1"},
            "links": {"system": 12345},
        }
    }


@pytest.fixture()
def bridge_reset_json():
    """Define a response to PUT /base_stations/:id/reset."""
    return {
        "base_stations": {
            "id": 12345,
            "name": "My Bridge",
            "mode": "home",
            "hardware_id": "0x1234567890abcdef",
            "hardware_revision": 4,
            "firmware_version": {
                "wifi": "0.121.0",
                "wifi_app": "3.3.0",
                "silabs": "1.0.1",
            },
            "missing_at": None,
            "created_at": "2019-04-30T01:43:50.497Z",
            "updated_at": "2019-04-30T01:44:43.749Z",
            "system_id": 12345,
            "firmware": {"wifi": "0.121.0", "wifi_app": "3.3.0", "silabs": "1.0.1"},
            "links": {"system": 12345},
        }
    }


@pytest.fixture()
def bridge_update_json():
    """Define a response to PUT /base_stations/:id."""
    return {
        "base_stations": {
            "id": 12345,
            "name": "My Updated Name",
            "mode": "home",
            "hardware_id": "0x1234567890abcdef",
            "hardware_revision": 4,
            "firmware_version": {
                "wifi": "0.121.0",
                "wifi_app": "3.3.0",
                "silabs": "1.0.1",
            },
            "missing_at": None,
            "created_at": "2019-04-30T01:43:50.497Z",
            "updated_at": "2019-04-30T01:44:43.749Z",
            "system_id": 12345,
            "firmware": {"wifi": "0.121.0", "wifi_app": "3.3.0", "silabs": "1.0.1"},
            "links": {"system": 12345},
        }
    }



"""Define endpoints for interacting with systems (accounts)."""
from typing import Callable


class System:
    """Define an object to interact with all endpoints."""

    def __init__(self, request: Callable) -> None:
        """Initialize."""
        self._request: Callable = request

    async def async_all(self) -> list:
        """Get all systems."""
        resp: dict = await self._request("get", "systems")
        return resp["systems"]

    async def async_create(self, attributes: dict) -> dict:
        """Create a system with a specific attribute payload."""
        resp: dict = await self._request(
            "post", "systems", json={"systems": attributes}
        )
        return resp["systems"]

    async def async_delete(self, system_id: int) -> None:
        """Delete a system by ID."""
        await self._request("delete", f"systems/{system_id}")

    async def async_get(self, system_id: int) -> dict:
        """Get a system by ID."""
        resp: dict = await self._request("get", f"systems/{system_id}")
        return resp["systems"]

    async def async_update(self, system_id: int, new_attributes: dict) -> dict:
        """Update a system with a specific attribute payload."""
        resp: dict = await self._request(
            "put", f"systems/{system_id}", json={"systems": new_attributes}
        )
        return resp["systems"]
